# -*- coding: utf-8 -*-

from cleo import Application as BaseApplication
from cleo.inputs.input import Input
from cleo.exceptions import MissingArguments
from cleo.inputs.input_option import InputOption
from .proxy_command import ProxyCommand


class ApplicationException(Exception):
    pass


class Application(BaseApplication):
    def __init__(self, kernel):
        super(Application, self).__init__()
        self.__kernel = kernel
        self.__kernel.boot()
        self.__container = kernel.get_container()

    def run(self, input_=None, output_=None):
        self.monkeypatch_validation()
        return super(Application, self).run(input_, output_)

    def do_run(self, input_, output_):
        self.setup_application(input_)

        for command in self.__container.get('command.tag'):
            self.add_command_proxy(command['class_instance'], command['key'])

        return super(Application, self).do_run(input_, output_)

    def add_command_proxy(self, command, service_key):
        self.add(ProxyCommand(command, service_key, self.__container))

    def get_default_input_definition(self):
        definition = super(Application, self).get_default_input_definition()
        definition.add_option(InputOption('--env', None, InputOption.VALUE_OPTIONAL, 'Application environment'))
        definition.add_option(InputOption('--no-debug', None, InputOption.VALUE_IS_FLAG, 'Disable debugging'))
        return definition

    def setup_application(self, input_):
        # self.setup_environment(input_)
        self.setup_logging()

    def setup_logging(self):
        self.logger = self.__container['logger']

    def monkeypatch_validation(self):
        #  Temporary monkey patching
        def validate(self):
            missing_args = []
            given_args = self.get_arguments()
            for argument in self.definition.get_arguments():
                if argument.is_required() and \
                        given_args.get(argument.get_name()) is None:
                    missing_args.append(argument.get_name())
            if len(missing_args):
                raise MissingArguments(
                    'Missing required arguments: {}'.format(','.join(missing_args))
                )

            self.validate_arguments()
            self.validate_options()

        Input.validate = validate

    def render_exception(self, e, output_):
        super(Application, self).render_exception(e, output_)
        if self.logger and 'cleo.exceptions' not in e.__class__.__module__ and e.__class__ is not ApplicationException:
            self.logger.exception('Exception')



from cleo import Command


class ProxyCommand(Command):
    def __init__(self, command_class, service_key, container):
        self.__doc__ = command_class.__doc__
        super(ProxyCommand, self).__init__()
        self.__service_key = service_key
        self.__container = container

    def handle(self):
        command = self.__container[self.__service_key]
        command.input = self.input
        command.output = self.output
        return command.handle()

    def interact(self, input_, output_):
        # If needed implement here
        pass



from datetime import datetime, timedelta


class DateSpan(object):
    def __init__(self, start_date, end_date):
        self.start_date = start_date
        self.end_date = end_date

    def __repr__(self):
        return '[{}, {}]'.format(
            self.start_date.strftime('%Y-%m-%d') if self.start_date else '',
            self.end_date.strftime('%Y-%m-%d') if self.end_date else ''
        )


class DateHelper(object):
    @staticmethod
    def is_valid_date(value):
        result = True
        try:
            datetime.strptime(value, '%Y-%m-%d')
        except ValueError:
            result = False
        return result

    @staticmethod
    def split_date_interval(start_date, end_date, span_delta):
        spans = []
        actual_span = DateSpan(start_date, None)
        while actual_span.end_date is None or actual_span.end_date < end_date:
            next_date = actual_span.start_date + span_delta
            if next_date > end_date:
                next_date = end_date
            actual_span.end_date = next_date
            spans.append(actual_span)
            if actual_span.end_date < end_date:
                actual_span = DateSpan(
                    actual_span.end_date + timedelta(days=1),
                    None
                )
        return spans



from setuptools import setup
import phoopy.console

long_description = open('README.rst', 'r').read()

setup(
    name='phoopy-console',
    version=phoopy.console.__version__,
    packages=['phoopy', 'phoopy.console', 'phoopy.console.helper'],
    setup_requires=['wheel'],
    install_requires=[
        'phoopy-kernel>=1.1.3,<1.2.0',
        'cleo==0.6.8'
    ],
    description="Console library for phoopy framework",
    long_description=long_description,
    url='https://github.com/phoopy/phoopy-console',
    author='Phoopy',
    author_email='reisraff@gmail.com',
    license='MIT',
    classifiers=[
        'Operating System :: OS Independent',
        'Development Status :: 5 - Production/Stable',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ]
)



# -*- coding: utf-8 -*-

import re
import string
from collections import Counter


class StringHelper(object):
    @staticmethod
    def snake_case(name):
        s1 = re.sub('(.)([A-Z][a-z]+)', r'\1_\2', name)
        return re.sub('([a-z0-9])([A-Z])', r'\1_\2', s1).lower()

    @staticmethod
    def remove_suffix(name, suffix):
        return re.sub('(.)({})'.format(suffix), r'\1', name)

    @staticmethod
    def filter_printable_characters(text):
        accents = 'áéíóúÁÉÍÓÚ'
        extras = 'ñÑç'
        brasilian = 'ãàâçêíõôü'
        printable = accents + extras + string.printable + brasilian
        return ''.join(filter(lambda x: x in printable, text))

    @staticmethod
    def is_valid_word(word, stopwords):
        for stopword in stopwords:
            if word.lower() != stopword.lower():
                continue
            return False

        without_punctuation = word.translate(str.maketrans('', '', string.punctuation)).strip()

        if len(without_punctuation) == 0:
            return False

        return True

    @staticmethod
    def get_most_used_words(words, stopwords):
        valid_words = [word.lower() for word in words if StringHelper.is_valid_word(word, stopwords)]
        return dict(Counter(valid_words).most_common(5))




class OutputHelper:
    colors = {
        'default_foreground': 39,
        'black': 30,
        'red': 31,
        'green': 32,
        'yellow': 33,
        'blue': 34,
        'magenta': 35,
        'cyan': 36,
        'light_gray': 37,
        'dark_gray': 90,
        'light_red': 91,
        'light_green': 92,
        'light_yellow': 93,
        'light_blue': 94,
        'light_magenta': 95,
        'light_cyan': 96,
        'white': 97,
    }

    @staticmethod
    def colorize(text, color):
        if color in OutputHelper.colors:
            return "\033[" + str(OutputHelper.colors[color]) + "m" + text + "\033[0m"

        return text



# -*- coding: utf-8 -*-

import logging
import os
import sys
import time
from datetime import datetime
from logging.handlers import RotatingFileHandler
from cleo import Command
from .helper import OutputHelper, StringHelper, DateHelper


class AbstractCommand(Command):
    def __init__(self, logger):
        super(AbstractCommand, self).__init__()
        self.logger = logger
        self.handler = None
        self._project_path = None

    def setup_logger(self, timestamp=False, name=None, max_bytes=1000000, backup_count=4):
        if not name:
            name = StringHelper.remove_suffix(StringHelper.snake_case(self.__class__.__name__), '_command')

        if timestamp:
            name = '{}-{}'.format(name, int(time.time()))

        filename = os.path.realpath(os.path.join(self._project_path, 'var', 'log', '{}.log'.format(name)))
        self.handler = RotatingFileHandler(
            filename=filename, maxBytes=max_bytes, backupCount=backup_count
        )
        formatter = logging.Formatter('[%(asctime)s] %(levelname)s %(message)s')
        self.handler.setFormatter(formatter)
        self.logger.addHandler(self.handler)
        sys.stdout.write('{}{}'.format(
            OutputHelper.colorize(
                'Logs are being writen to the file ',
                'yellow'
            ),
            OutputHelper.colorize(
                '"{}"\n\n'.format(filename),
                'cyan'
            )
        ))
        sys.stdout.flush()

    def get_date_option(self, key):
        input_date = self.option(key)
        if input_date and not DateHelper.is_valid_date(input_date):
            raise Exception('Invalid date format: {}, must be Y-m-d'.format(input_date))
        return datetime.strptime(input_date, '%Y-%m-%d').date() if input_date else None

    @property
    def project_path(self):
        return self._project_path

    @project_path.setter
    def project_path(self, project_path):
        self._project_path = project_path

    def validate_setup(self):
        if not self.handler:
            self.setup_logger(True)

    def info(self, message):
        self.validate_setup()
        self.logger.info(message)

    def success(self, message):
        self.validate_setup()
        self.logger.info(OutputHelper.colorize(message, 'green'))

    def comment(self, message):
        self.validate_setup()
        self.logger.info(OutputHelper.colorize(message, 'yellow'))

    def question(self, message):
        self.validate_setup()
        self.logger.info(OutputHelper.colorize(message, 'blue'))

    def error(self, message):
        self.validate_setup()
        self.logger.info(OutputHelper.colorize(message, 'red'))

    def line(self, message):
        pass
